package xyz.sheepstar.wrapper.core;

import com.google.common.reflect.ClassPath;
import drivers.MySQLConnection;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.JDABuilder;
import net.dv8tion.jda.api.requests.GatewayIntent;
import org.apache.commons.cli.*;
import types.LogLevelType;
import xyz.sheepstar.wrapper.util.Config;
import xyz.sheepstar.wrapper.util.api.module.SheepstarModule;

import javax.security.auth.login.LoginException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Set;

/********************************
 * @author Mathias Wagner
 * Created 23.01.2021
 ********************************/

public class Bootstrap {

    private String[] args = null;
    private Config config = null;
    private CommandLine commandLine = null;
    private MySQLConnection mySQLConnection = null;
    private JDA jda;

    public Bootstrap(String[] args) {
        this.args = args;
    }

    public Bootstrap init() {
        validateArguments();
        loadConfiguration();
        registerMySQL();
        registerJDA();
        return this;
    }

    private void validateArguments() {
        Options options = new Options();

        Option input = new Option("cf", "configFile", true, "Sheepstar Config File");
        input.setRequired(true);
        options.addOption(input);

        CommandLineParser parser = new DefaultParser();
        HelpFormatter formatter = new HelpFormatter();
        try {
            this.commandLine = parser.parse(options, this.args);
        } catch (ParseException e) {
            System.out.println(e.getMessage());
            formatter.printHelp("sheepstar-bot", options);
            System.exit(1);
        }
    }

    private void loadConfiguration() {
        this.config = new Config(this.commandLine.getOptionValue("configFile"));
    }

    private void registerMySQL() {
        this.mySQLConnection = new MySQLConnection(
                config.getString("mysql_host"),
                config.getString("mysql_user"),
                config.getString("mysql_pass"),
                config.getString("mysql_data"),
                LogLevelType.ALL
        ).connect();
    }

    private void registerJDA() {
        try {
            jda = JDABuilder.createDefault(config.getString("bot_token")).enableIntents(GatewayIntent.GUILD_MEMBERS).build();
        } catch (LoginException e) {
            System.out.println(e.getMessage());
            System.exit(0);
        }
    }

    public void enableModule(String module) {
        try {
            ((SheepstarModule) Class.forName("xyz.sheepstar.wrapper.modules."+ module.toLowerCase()+".Bootstrap").newInstance()).init();
            System.out.println("Modul '" + module + "' erfolgreich geladen.");
        } catch (Exception e) {System.out.println("Modul '"+ module.toLowerCase() +"' konnte nicht geladen werden: " + e.getMessage()); }
    }

    public void enableModules(String... modules) {
        for (String module : modules) enableModule(module);
    }

    public void enableAllModules(String moduleRoot) {
        ArrayList<String> classes = new ArrayList<>();
        try {
            ClassLoader cl = getClass().getClassLoader();
            Set<ClassPath.ClassInfo> classesInPackage = ClassPath.from(cl).getTopLevelClassesRecursive(moduleRoot);
            classesInPackage.forEach(info -> {
                String className = info.getPackageName().replaceAll(moduleRoot+".", "");
                if (className.split("\\.").length != 1) className = className.split("\\.")[0];
                if (!classes.contains(className)) classes.add(className);
            });
        } catch (IOException e) {  }
        classes.forEach(className -> {
            try { enableModule(className); } catch (Exception ignored) {}
        });
        System.out.println("Laden der Module abgeschlossen");
    }

    public JDA getJDA() {
        return jda;
    }

    public CommandLine getArgs() {
        return commandLine;
    }

    public Config getConfig() {
        return config;
    }

    public MySQLConnection getMySQLConnection() {
        return mySQLConnection;
    }
}
